﻿var portConnected = null;
var emulatorConnected = false;
var xslPorts = loadXML("ports.xsl");
var xmlPids = loadXML("pids.xml");
var pidDefaults;
var groups = xmlPids.getElementsByTagName("group");
var ignition = true;

// readiness monitors (PID 0101)
var readiness = new Array(0, 0x7, 0xff, 0);

transformXML(xmlPids, loadXML("tabs.xsl"), "tabs");
transformXML(xmlPids, loadXML("knobs.xsl"), "panel");

function onLoad() {
    var xmlDefaults = loadXML("defaults.xml");
    if (xmlDefaults) pidDefaults = xmlDefaults.getElementsByTagName("param");
    self.setTimeout(autoConnect, 1000);
}

function listSerialPorts() {
    transformXML(loadXML("/serial"), xslPorts, "ports");
}

function chooseFile() {
	var path = httpGet('/choosefile');
	if (path) {
		filepath = path;
		LoadCode(filepath);
		return path;
	}
	return null;
}

function findEmulator() {
    var xmlSerialPorts = loadXML("/serial");
    var ports = xmlSerialPorts.getElementsByTagName("Port");
    var portFound = null;
    for (var n = ports.length - 1; n >= 0; n--) {
        portFound = ports[n].firstChild.nodeValue;
		var success;
		if (portConnected = SerialOpen(portFound, 38400)) {
		    var s = sendCommand("ATINF0");
		    success = sendCommand(ignition ? "ATACC1" : "ATACC0");
        }
        SerialClose();
		portConnected = false;
        if (success) {
            if (connect(portFound)) {
				// connection completed
				clearDTC(true);
                break;
			}
        }
        portFound = null;
    }
    if (!portFound) {
        alert("Freematics OBD-II Emulator is not connected.\nPlease connect it to the PC via USB cable.\n\nFor product information, please visit http://freematics.com");
        disconnect();
    }
}

function autoConnect()
{
    if (portConnected) {
        disconnect();
        return;
    }
	emulatorConnected = false;
    document.getElementById("button_conn").innerHTML = "Connecting...";
    self.setTimeout(findEmulator, 10);
}

function byte2hex(n)
{
	return n >= 0x10 ? n.toString(16) : ("0" + n.toString(16));
}

function pushReadiness()
{
	var cmd = "ATSET 0101=0x" + byte2hex(readiness[0]) + ",0x" + byte2hex(readiness[1]) + ",0x" + byte2hex(readiness[2]) + ",0x" + byte2hex(readiness[3]);
    return sendCommand(cmd);
}

function init() {
    if (!sendCommand("ATE0") && !pushReadiness()) {
        alert("Error occurred during initialization");
        disconnect();
        return false;
    }
	showReadiness();

    document.getElementById("status").className = "top_connected";
	document.getElementById("button_conn").innerHTML = "Disconnect";
	document.getElementById("button_ignite").disabled = false;

    /*
    var result = SerialWrite("ATZ\r", "%0D", 10000);
    if (!result) {
        alert("Error occurred during initialization.");
        disconnect();
        return;
    }
    */

    pushDefaultPidValue(0);	
}

function connect(port) {
	if (!SerialOpen(port, 38400)) {
		alert("Unable to connect via " + port);
		return false;
	}

	portConnected = port;
	self.setTimeout(init, 10);
	return true;
}

function disconnect() {
	SerialClose();
	portConnected = null;
	document.getElementById("status").className = "top";
	document.getElementById("button_conn").innerHTML = "Connect";
	document.getElementById("button_ignite").disabled = true;
}

function sendCommand(cmd) {
    if (!portConnected) {
        return false;
    }
    return SerialWrite(cmd + "\r", "OK%0D");
}

var dtcCount = [];
var dtcCodes = [];

function setDTC(mode, code)
{
	if (dtcCount[mode] >= 6) {
		alert("Up to 6 DTC can bet set for a mode. To set additional DTC, please clear previous ones first.");
		return;
	}
	if (dtcCodes[mode].indexOf(code) >= 0) {
		alert("DTC " + code + " was already set for Mode 0" + mode.toString(16));
		return;
	}
	
	var codes = dtcCodes[mode];
	if (codes != "") codes += ",";
	codes += code;
	if (!sendCommand("ATSET DTC=" + codes)) {
		alert("Unable to set DTC " + code);
	} else {
		dtcCount[mode]++;
		dtcCodes[mode] = codes;
		alert("DTC " + code + " for Mode 0" +  mode.toString(16) + " has been set successfully.\nTotally " + dtcCount[mode] + " DTCs has been set");
	}
}

function clearDTC(quiet)
{
    if (!sendCommand("ATCLR DTC")) {
        if (!quiet) alert("Error clearing DTC");
    } else {
        if (!quiet) alert("DTC cleared successfully");
		dtcCount[0x3] = 0;
		dtcCount[0x7] = 0;
		dtcCount[0xa] = 0;
		dtcCodes[0x3] = "";
		dtcCodes[0x7] = "";
		dtcCodes[0xa] = "";
    }
}

function setVIN(vin)
{
    if (!sendCommand("ATSET VIN=" + vin)) {
        alert("Unable to set VIN");
    } else {
        alert("VIN set to " + vin);
    }
}

function setProtocol(protocol)
{
    if (!sendCommand("ATBUS " + protocol)) {
        alert("Unable to set the protocol to " + protocol + "\nMake sure the protocol is supported by the emulator.");
    } else {
		var isJ1850 = protocol.substr(0, 5) == "J1850";
		document.getElementById("set_dtc_07").disabled = !isJ1850;
		document.getElementById("set_dtc_0a").disabled = !isJ1850;
        alert("Protocol set successfully");
    }
}

function updateFirmware()
{
	alert("Please be aware there are two types of Freematics OBD-II Emulator (with and without J1850 support). Flashing a wrong firmware file will likely brick your device.")
    httpGet("/navigate?url=FirmwareUpdater.exe&arg=" + portConnected);
    disconnect();
}

function replayLogFile()
{
	alert("This feature is not implemented yet");
}

function pushDefaultPidValue(index) {
    if (index < pidDefaults.length && portConnected) {
        var pid = pidDefaults[index].getAttribute("id");
        var value = pidDefaults[index].getAttribute("value");
        if (!sendCommand("ATSET " + pid + "=" + value)) {
            alert("Error setting PID " + pid);
            return;
        }
        for (m = 0; m < groups.length; m++) {
            var e = document.getElementById(groups[m].getAttribute("name") + "-" + pid);
            if (e) {
                e.value = value;
            }
        }
        self.setTimeout("pushDefaultPidValue(" + (index + 1) + ")", 0);
    } else {
		// done
		emulatorConnected = true;
	}
}

function SetPidValue(pid, value)
{
    if (!SerialWrite("ATSET " + pid + "=" + value + "\r")) {
        alert("Emulator not connected");
        return false;
    }
	SerialRead();
    return true;
}

function toggleIgnition()
{
    if (!sendCommand(ignition ? "ATACC0" : "ATACC1")) {
        alert("Unable to set ignition status");
        return false;
    }
	ignition = !ignition;
	var e = document.getElementById("button_ignite");
	e.innerHTML = "Ignition is " + (ignition ? "ON" : "OFF");
	e.className = ignition ? "ignite_button" : "ignite_button_off";
    return true;
}

var pidElement;
var pidDiff;
var pidInterval;
var pidValueMin;
var pidValueMax;
var pidActive;

function StartChangeValue(element, pid, diff, minval, maxval)
{
    pidActive = pid;
    pidElement = element;
	pidDiff = diff;
	pidInterval = 200;
	pidValueMin = minval;
	pidValueMax = maxval;
	timerChangeValue();
}

function timerChangeValue()
{
	if (pidElement) {
		var val = parseInt(pidElement.value);
		val += pidDiff;
		if (val < pidValueMin)
		    val = pidValueMin;
		else if (val > pidValueMax)
		    val = pidValueMax;
		
		if (SetPidValue(pidActive, val) && pidDiff != 0) {
   		    pidElement.value = val;
   		    if (pidInterval > 50)
   		        pidInterval -= 10;
   		    if (pidInterval) self.setTimeout(timerChangeValue, pidInterval);
		}
	}
}

function StopChangeValue()
{
    pidDiff = 0;
    pidInterval = 0;
    timerChangeValue();
    pidElement = null;
}

var pidDisabled = new Array(256);

function togglePID(pid, enabled)
{
	var i = parseInt(pid, 16) & 0xff;
	pidDisabled[i] = !enabled;
	var specialPid = i & 0xE0;
	var val = 0;
	var vals = "";
	var pidTmp = specialPid + 1;
	for (m = 0; m < 4; m++) {
		val = 0;
		for (n = 1; n <= 8; n++) {
			val <<= 1;
			if (!pidDisabled[pidTmp++]) val |= 1;
		}
		vals += val.toString(16) + ",";
	}
	specialPid |= 0x100;
	var cmd = "ATSET 0" + specialPid.toString(16) + "=" + vals.substr(0, vals.length - 1);
    if (!sendCommand(cmd)) {
        alert("Unable to set PID " + specialPid.toString(16));
        return false;
    } else {
		alert("PID " + pid + " is " + (enabled ? "enabled" : "disabled"));
	}
}

function toggleVIN(enabled)
{
	if (sendCommand(enabled ? "ATVIN1" : "ATVIN0")) {
		alert("VIN response is " + (enabled ? "enabled" : "disabled"));
	} else {
		alert("Error toggling VIN response");
	}
}

function getReadinessBit(code)
{
	var byte = 0;
	switch (code.substr(0, 1)) {
	case 'A':
		byte = readiness[0];
		break;
	case 'B':
		byte = readiness[1];
		break;
	case 'C':
		byte = readiness[2];
		break;
	case 'D':
		byte = readiness[3];
		break;
	}
	return (byte & (1 << parseInt(code.substr(1, 1)))) != 0;
}

function setReadinessBit(code, set)
{
	var bit = 1 << parseInt(code.substr(1, 1));
	switch (code.substr(0, 1)) {
	case 'A':
		if (set)
			readiness[0] |= bit;
		else 
			readiness[0] &= ~bit;
		break;
	case 'B':
		if (set)
			readiness[1] |= bit;
		else 
			readiness[1] &= ~bit;
		break;
	case 'C':
		if (set)
			readiness[2] |= bit;
		else 
			readiness[2] &= ~bit;
		break;
	case 'D':
		if (set)
			readiness[3] |= bit;
		else 
			readiness[3] &= ~bit;
		break;
	}
}

function showReadiness()
{
	var list;
	for (m = 0; m < groups.length; m++) {
		if (groups[m].getAttribute("name") == "Readiness") {
			list = groups[m].getElementsByTagName("readiness");
		}
	}
	if (!list) return;
	for (var i = 0; i < list.length; i++) {
		var id = list[i].getAttribute("id");
		document.getElementById("Availability-" + id).setAttribute("checked", getReadinessBit(list[i].getAttribute("available")) ? "checked" : "");
		document.getElementById("Readiness-" + id).innerHTML = getReadinessBit(list[i].getAttribute("complete")) ? "Incomplete" : "Complete";
	}
	document.getElementById("DTC_COUNT").value = readiness[0] & 0x7f;
}

function setDTCCount(value)
{
	if (value >= 0 && value <= 127) {
		var MIL = value > 0 ? 0x80 : 0;
		readiness[0] = MIL | value;
		if (pushReadiness()) 
			document.getElementById("DTC_COUNT").value = value;
	}
}

function setAvailability(code, set)
{
	setReadinessBit(code, set);
	return pushReadiness();	
}

function setCompleteness(code, set)
{
	setReadinessBit(code, set);
	if (pushReadiness()) {
		return set ? "Incomplete" : "Complete";
	} else {
		return "";
	}	
}